-- Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

entity DVI_CTRL is
	port(
		RST_N		: in  std_logic;
		DCLK : in std_logic;
		SCLK : in std_logic;

		disp_ena		: in  std_logic;
		disp_b		: in  std_logic_vector(7 downto 0);
		disp_g		: in  std_logic_vector(7 downto 0);
		disp_r		: in  std_logic_vector(7 downto 0);
		disp_hsync	: in  std_logic;
		disp_vsync	: in  std_logic;
		disp_control		: in  std_logic_vector(3 downto 0) :="0000";

		data0p_h	: out std_logic;
		data0p_l	: out std_logic;
		data0n_h	: out std_logic;
		data0n_l	: out std_logic;

		data1p_h	: out std_logic;
		data1p_l	: out std_logic;
		data1n_h	: out std_logic;
		data1n_l	: out std_logic;

		data2p_h	: out std_logic;
		data2p_l	: out std_logic;
		data2n_h	: out std_logic;
		data2n_l	: out std_logic;

		clockp_h	: out std_logic;
		clockp_l	: out std_logic;
		clockn_h	: out std_logic;
		clockn_l	: out std_logic
	);
end DVI_CTRL;

architecture RTL of DVI_CTRL is

	component tmds_encoder
	port(
		clk      : IN  STD_LOGIC;                     
		disp_ena : IN  STD_LOGIC;                     
		control  : IN  STD_LOGIC_VECTOR(1 DOWNTO 0);  
		d_in     : IN  STD_LOGIC_VECTOR(7 DOWNTO 0);  
		q_out    : OUT STD_LOGIC_VECTOR(9 DOWNTO 0)); 
	end component;


	component dvi_transmitter
	port(
		RST_N		: in  std_logic;
		DCLK : in std_logic;
		SCLK : in std_logic;
    
		data0_in	: in  std_logic_vector(9 downto 0);
		data1_in	: in  std_logic_vector(9 downto 0);
		data2_in	: in  std_logic_vector(9 downto 0);
		
		data0p_h	: out std_logic;
		data0p_l	: out std_logic;
		data0n_h	: out std_logic;
		data0n_l	: out std_logic;

		data1p_h	: out std_logic;
		data1p_l	: out std_logic;
		data1n_h	: out std_logic;
		data1n_l	: out std_logic;

		data2p_h	: out std_logic;
		data2p_l	: out std_logic;
		data2n_h	: out std_logic;
		data2n_l	: out std_logic;

		clockp_h	: out std_logic;
		clockp_l	: out std_logic;
		clockn_h	: out std_logic;
		clockn_l	: out std_logic
	);
	end component;

	signal disp_vh	: std_logic_vector(1 downto 0);

	signal q_b	: std_logic_vector(9 downto 0);
	signal q_g	: std_logic_vector(9 downto 0);
	signal q_r	: std_logic_vector(9 downto 0);


begin
  	tmds_encoder_r : tmds_encoder
	port map (
		clk      => DCLK,  --system clock
		disp_ena => disp_ena,  --display enable
		control  => disp_control(3 downto 2),  --C3, C2
		d_in     => disp_r,  --8-bit data input
		q_out   => q_r  --10-bit encoder output
	);

	tmds_encoder_g : tmds_encoder
	port map (
		clk      => DCLK,  --system clock
		disp_ena => disp_ena,  --display enable
		control  => disp_control(1 downto 0),  --C1, C0
		d_in     => disp_g,  --8-bit data input
		q_out   => q_g  --10-bit encoder output
	);
	
	tmds_encoder_b : tmds_encoder
	port map (
		clk      => DCLK,  --system clock
		disp_ena => disp_ena,  --display enable
		control  => disp_vh,  --vsync, hsync
		d_in     => disp_b,  --8-bit data input
		q_out   => q_b  --10-bit encoder output
	);
	
	disp_vh <= disp_vsync & disp_hsync;

	dvi_transmitter_1 : dvi_transmitter
	port map (
		RST_N		=> RST_N,
		DCLK => DCLK,
		SCLK => SCLK,
    
		data0_in	=> q_b,
		data1_in	=> q_g,
		data2_in	=> q_r,
		
		data0p_h	=> data0p_h,
		data0p_l	=> data0p_l,
		data0n_h	=> data0n_h,
		data0n_l	=> data0n_l,

		data1p_h	=> data1p_h,
		data1p_l	=> data1p_l,
		data1n_h	=> data1n_h,
		data1n_l	=> data1n_l,

		data2p_h	=> data2p_h,
		data2p_l	=> data2p_l,
		data2n_h	=> data2n_h,
		data2n_l	=> data2n_l,

		clockp_h	=> clockp_h,
		clockp_l	=> clockp_l,
		clockn_h	=> clockn_h,
		clockn_l	=> clockn_l

	);

end RTL;
