-- Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

LIBRARY ieee;
USE ieee.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

ENTITY LINEOUT_CTRL IS
	PORT	(
		CLK	: IN std_logic;
		CLK100M	: IN std_logic;
		RST_N	: IN std_logic;
		latch0	: IN std_logic_vector(15 downto 0);
		latch1	: IN std_logic_vector(15 downto 0);
		latch2	: IN std_logic_vector(15 downto 0);
		latch3	: IN std_logic_vector(15 downto 0);
		CamLineCount : in std_logic_vector(8 downto 0);
		VgaLineCount	: in std_logic_vector(8 downto 0);
		VgaPixCount : IN std_logic_vector(9 downto 0);
  vga_RGB_latch0 : out std_logic_vector(15 downto 0);
  vga_RGB_latch1 : out std_logic_vector(15 downto 0);
  vga_RGB_latch2 : out std_logic_vector(15 downto 0);
  vga_RGB_latch3 : out std_logic_vector(15 downto 0);
		winp0: in std_logic;
		winp1: in std_logic;
		winp2: in std_logic;
		winp3: in std_logic;
  vga_r : out std_logic_vector(7 downto 0);
  vga_g : out std_logic_vector(7 downto 0);
  vga_b : out std_logic_vector(7 downto 0)
 );
END LINEOUT_CTRL;

ARCHITECTURE structure OF LINEOUT_CTRL IS

component SRAM
   Port (
          CLK : In std_logic;
          CS_N : In std_logic;
          WR_N : In std_logic;
          WRADDR : In std_logic_vector(9 downto 0);
          RDADDR : In std_logic_vector(9 downto 0);
          WRDATA : In std_logic_vector(15 downto 0);
          RDDATA : Out std_logic_vector(15 downto 0) );
end component;

 signal OB_WR_ADDR : std_logic_vector(9 downto 0);
 signal OB_RD_ADDR : std_logic_vector(9 downto 0);
 signal OB_WR_DATA : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_A : std_logic_vector(15 downto 0);
 signal OB_RD_DATA_B : std_logic_vector(15 downto 0);
 signal OB_WR_N_A : std_logic;
 signal OB_WR_N_B : std_logic;
 signal OB_WR_ADDR_sig : std_logic_vector(9 downto 0);
 signal OB_RD_ADDR_sig : std_logic_vector(9 downto 0);
 signal OB_WR_DATA_sig : std_logic_vector(15 downto 0);
 signal OB_WR_N_A_sig : std_logic;
 signal OB_WR_N_B_sig : std_logic;
 signal OB_CS_N : std_logic;
signal ODDLINE, oddline_dly1, oddline_dly2 : std_logic;
 signal ls2b : std_logic_vector(1 downto 0);
signal  vga_RGB : std_logic_vector(15 downto 0);
 signal LB_ADDR : std_logic_vector(9 downto 0);

BEGIN

-------------------------------------------------------------------
OutBuf_A : SRAM port map (
   CLK		=> CLK,
   CS_N 	=> OB_CS_N,
   WR_N	=> OB_WR_N_A, 
   WRADDR	=> OB_WR_ADDR, 
   RDADDR	=> OB_RD_ADDR, 
   WRDATA	=> OB_WR_DATA, 
   RDDATA	=> OB_RD_DATA_A );

-------------------------------------------------------------------
OutBuf_B : SRAM port map (
   CLK		=> CLK,
   CS_N 	=> OB_CS_N,
   WR_N	=> OB_WR_N_B, 
   WRADDR	=> OB_WR_ADDR, 
   RDADDR	=> OB_RD_ADDR, 
   WRDATA	=> OB_WR_DATA, 
   RDDATA	=> OB_RD_DATA_B );

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     oddline_dly2 <= '0';
     oddline_dly1 <= '0';
   elsif( CLK'event and CLK = '1' ) then
     oddline_dly2 <= oddline_dly1;
     oddline_dly1 <= ODDLINE;
   end if;  
end process;

  ODDLINE <= VgaLineCount(1);

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     vga_RGB <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(oddline_dly2 = '1' ) then
       vga_RGB <= OB_RD_DATA_A;
     else 
       vga_RGB <= OB_RD_DATA_B;
     end if;  
  end if;
end process;

-------------------------------------------------------------------
  OB_WR_N_A_sig <= '0' when (oddline_dly2 = '0' and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1')) else '1';
                       
  OB_WR_N_B_sig <= '0' when (oddline_dly2 = '1' and 
                     (winp0 = '1' or winp1 = '1' or winp2 = '1' or winp3 = '1')) else '1'; 
  
  OB_WR_DATA_sig <= latch0(15 downto 0) when winp0 = '1' else
                   latch1(15 downto 0) when winp1 = '1' else
                   latch2(15 downto 0) when winp2 = '1' else latch3(15 downto 0);
  
-------------------------------------------------------------------
  ls2b <= "00" when winp0 = '1' else
          "01" when winp1 = '1' else
          "10" when winp2 = '1' else
          "11";

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
	if( RST_N = '0' ) then
		LB_ADDR <= (others => '0');
	elsif( CLK'event and CLK = '1' ) then
		LB_ADDR <= VgaPixCount(9 downto 0) + 20;
	end if;  
end process;
          
-------------------------------------------------------------- 
  OB_WR_ADDR_sig <= LB_ADDR(9 downto 2) & ls2b;
  OB_RD_ADDR_sig <= LB_ADDR;
  OB_CS_N <= '0';

  vga_r <= vga_RGB(15 downto 11) & "000";
  vga_g <= vga_RGB(10 downto 5) & "00";
  vga_b <= vga_RGB(4 downto 0) & "000";
  
-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     OB_RD_ADDR <= (others => '0');
     OB_WR_ADDR <= (others => '0');
     OB_WR_DATA <= (others => '0');
     OB_WR_N_A <= '0';
     OB_WR_N_B <= '0';
   elsif( CLK'event and CLK = '1' ) then
     OB_RD_ADDR <= OB_RD_ADDR_sig;
     OB_WR_ADDR <= OB_WR_ADDR_sig;
     OB_WR_DATA <= OB_WR_DATA_sig;
     OB_WR_N_A <= OB_WR_N_A_sig;
     OB_WR_N_B <= OB_WR_N_B_sig;
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     vga_RGB_latch0 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(OB_RD_ADDR(1 downto 0) = 0 ) then
       vga_RGB_latch0 <= vga_RGB;
     end if;  
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     vga_RGB_latch1 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(OB_RD_ADDR(1 downto 0) = 1 ) then
       vga_RGB_latch1 <= vga_RGB;
     end if;  
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     vga_RGB_latch2 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(OB_RD_ADDR(1 downto 0) = 2 ) then
       vga_RGB_latch2 <= vga_RGB;
     end if;  
  end if;
end process;

-------------------------------------------------------------- 
process( RST_N, CLK )  
begin 
   if( RST_N = '0' ) then
     vga_RGB_latch3 <= (others => '0');
   elsif( CLK'event and CLK = '1' ) then
     if(OB_RD_ADDR(1 downto 0) = 3 ) then
       vga_RGB_latch3 <= vga_RGB;
     end if;  
  end if;
end process;

END structure;
