-- Test Bench
-- Designed by T.Iwata at DIGITALFILTER.COM 2016/03/29

library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.STD_LOGIC_ARITH.ALL;

entity tb_VideoProc is
end tb_VideoProc ;

architecture beh of tb_VideoProc is

component VideoProcCore
	Port (
	CLK	: IN std_logic;
	CLK100M	: IN std_logic;
	RST_N	: IN std_logic;
	XCLK : out std_logic;
	CamHsync	: IN std_logic;
	CamVsync	: IN std_logic;
	PCLK	: IN std_logic;
  CamData	: in std_logic_vector(7 downto 0);
	SDRAM_DQENB_N : out std_logic;
  SDRAM_DQMH	: out std_logic;
  SDRAM_DQML	: out std_logic;
  SDRAM_CS_N	: out std_logic;
  SDRAM_WE_N	: out std_logic;
  SDRAM_RAS_N	: out std_logic;
  SDRAM_CAS_N	: out std_logic;
  SDRAM_CKE	: out std_logic;
  SDRAM_DQI : in std_logic_vector(15 downto 0);
  SDRAM_DQO : out std_logic_vector(15 downto 0);
  SDRAM_A : out std_logic_vector(11 downto 0);
  SDRAM_BA : out std_logic_vector(1 downto 0);
  SDRAM_CLK	: out std_logic;
  		DCLK : in std_logic;
		SCLK : in std_logic;

		data0p_h	: out std_logic;
		data0p_l	: out std_logic;
		data0n_h	: out std_logic;
		data0n_l	: out std_logic;

		data1p_h	: out std_logic;
		data1p_l	: out std_logic;
		data1n_h	: out std_logic;
		data1n_l	: out std_logic;

		data2p_h	: out std_logic;
		data2p_l	: out std_logic;
		data2n_h	: out std_logic;
		data2n_l	: out std_logic;

		clockp_h	: out std_logic;
		clockp_l	: out std_logic;
		clockn_h	: out std_logic;
		clockn_l	: out std_logic );
end component;

component mt48lc4m16  
	Port (
        BA0       : IN    std_logic := 'U';
        BA1       : IN    std_logic := 'U';
        DQML      : IN    std_logic := 'U';
        DQMH      : IN    std_logic := 'U';
        DQ0       : INOUT std_logic := 'U';
        DQ1       : INOUT std_logic := 'U';
        DQ2       : INOUT std_logic := 'U';
        DQ3       : INOUT std_logic := 'U';
        DQ4       : INOUT std_logic := 'U';
        DQ5       : INOUT std_logic := 'U';
        DQ6       : INOUT std_logic := 'U';
        DQ7       : INOUT std_logic := 'U';
        DQ8       : INOUT std_logic := 'U';
        DQ9       : INOUT std_logic := 'U';
        DQ10      : INOUT std_logic := 'U';
        DQ11      : INOUT std_logic := 'U';
        DQ12      : INOUT std_logic := 'U';
        DQ13      : INOUT std_logic := 'U';
        DQ14      : INOUT std_logic := 'U';
        DQ15      : INOUT std_logic := 'U';
        CLK       : IN    std_logic := 'U';
        CKE       : IN    std_logic := 'U';
        A0        : IN    std_logic := 'U';
        A1        : IN    std_logic := 'U';
        A2        : IN    std_logic := 'U';
        A3        : IN    std_logic := 'U';
        A4        : IN    std_logic := 'U';
        A5        : IN    std_logic := 'U';
        A6        : IN    std_logic := 'U';
        A7        : IN    std_logic := 'U';
        A8        : IN    std_logic := 'U';
        A9        : IN    std_logic := 'U';
        A10       : IN    std_logic := 'U';
        A11       : IN    std_logic := 'U';
        WENeg     : IN    std_logic := 'U';
        RASNeg    : IN    std_logic := 'U';
        CSNeg     : IN    std_logic := 'U';
        CASNeg    : IN    std_logic := 'U'
	);
end component;

component env
	Port (
          CLK : out std_logic; 
          CLK100 : out std_logic; 
          RST_N : out std_logic;
          PCLK : out std_logic;
          DCLK : out std_logic;
          SCLK : out std_logic;
          CamHsync : out std_logic;
          CamVsync : out std_logic;
          CamData : Out std_logic_vector(7 downto 0) );
end component;


signal xclk_sig : std_logic;
signal rst_n_sig : std_logic;
signal mclk_sig, mclk100_sig : std_logic;
signal fsclk_sig : std_logic;
signal packet_sig : std_logic;
signal strobe_sig : std_logic;
signal yin_sig : std_logic_vector(7 downto 0);
signal dummy_high : std_logic;
signal sdata_sig : std_logic;
signal latch0_sig : std_logic;
signal trigger_sig : std_logic;
signal CamHsync_sig, CamVsync_sig, pclk_sig : std_logic;

signal  SDRAM_DQENB_N : std_logic;
signal   SDRAM_DQMH	: std_logic;
signal   SDRAM_DQML	: std_logic;
signal   SDRAM_CS_N	: std_logic;
signal   SDRAM_WE_N	: std_logic;
signal   SDRAM_RAS_N	: std_logic;
signal   SDRAM_CAS_N	: std_logic;
signal   SDRAM_CKE	: std_logic;
signal   SDRAM_CLK : std_logic;
signal   SDRAM_DQI, SDRAM_DQO : std_logic_vector(15 downto 0);
signal   SDRAM_A : std_logic_vector(11 downto 0);
signal   SDRAM_BA : std_logic_vector(1 downto 0);
signal  SDRAM_DQ_ENB : std_logic;
signal dq : std_logic_vector(15 downto 0);
signal wen : std_logic;
signal rasn : std_logic;
signal csn : std_logic;
signal casn : std_logic;
signal daddr : std_logic_vector(11 downto 0);
signal dq_out_dly : std_logic_vector(15 downto 0);
signal pix2up_sig, pix2logic_sig : std_logic_vector(15 downto 0);
signal pixstb_sig, pixrd_sig, pixwr_sig : std_logic;
signal dvi_data0_p, dvi_data0_n		: std_logic;		-- GPIO1_D17
signal dvi_data1_p, dvi_data1_n		: std_logic;		-- GPIO1_D21
signal dvi_data2_p, dvi_data2_n		: std_logic;		-- GPIO1_D12
signal dvi_clock_p, dvi_clock_n		: std_logic;		-- GPIO1_D9
signal DCLK, SCLK : std_logic;
signal data0p_h, data0p_l, data0n_h, data0n_l : std_logic;	
signal data1p_h, data1p_l, data1n_h, data1n_l : std_logic;
signal data2p_h, data2p_l, data2n_h, data2n_l : std_logic;
signal clockp_h, clockp_l, clockn_h, clockn_l : std_logic;

begin
  
  dummy_high <= '1';
  
VideoProcCore_1 : VideoProcCore port map (
		CLK	=> mclk_sig,
		CLK100M	=> mclk100_sig,
		RST_N	=> rst_n_sig,
		XCLK	=> xclk_sig,
	CamHsync	=> CamHsync_sig,
	CamVsync	=> CamVsync_sig,
	PCLK	=> pclk_sig,
  CamData	=> yin_sig,
     SDRAM_DQENB_N => SDRAM_DQENB_N,
  SDRAM_DQMH	=> SDRAM_DQMH,
  SDRAM_DQML	=> SDRAM_DQML,
  SDRAM_CS_N	=> SDRAM_CS_N,
  SDRAM_WE_N	=> SDRAM_WE_N,
  SDRAM_RAS_N	=> SDRAM_RAS_N,
  SDRAM_CAS_N	=> SDRAM_CAS_N,
  SDRAM_CKE	=> SDRAM_CKE,
  SDRAM_DQI => SDRAM_DQI,
  SDRAM_DQO => SDRAM_DQO,
  SDRAM_A => SDRAM_A,
  SDRAM_BA => SDRAM_BA,
  SDRAM_CLK	=> SDRAM_CLK,
		DCLK => DCLK,
		SCLK => SCLK,

	data0p_h	=> data0p_h,
	data0p_l	=> data0p_l,
	data0n_h	=> data0n_h,
	data0n_l	=> data0n_l,

	data1p_h	=> data1p_h,
	data1p_l	=> data1p_l,
	data1n_h	=> data1n_h,
	data1n_l	=> data1n_l,

	data2p_h	=> data2p_h,
	data2p_l	=> data2p_l,
	data2n_h	=> data2n_h,
	data2n_l	=> data2n_l,

	clockp_h	=> clockp_h,
	clockp_l	=> clockp_l,
	clockn_h	=> clockn_h,
	clockn_l	=> clockn_l );

mt48lc4m16_1 : mt48lc4m16 port map (
        BA0             => SDRAM_BA(0),
        BA1             => SDRAM_BA(1),
        DQML             => SDRAM_DQML,
        DQMH             => SDRAM_DQMH,
        DQ0             => dq(0), 
        DQ1             => dq(1),
        DQ2             => dq(2),
        DQ3             => dq(3),
        DQ4             => dq(4),
        DQ5             => dq(5),
        DQ6             => dq(6),
        DQ7             => dq(7),
        DQ8             => dq(8),
        DQ9             => dq(9),
        DQ10            => dq(10),
        DQ11            => dq(11),
        DQ12            => dq(12),
        DQ13            => dq(13),
        DQ14            => dq(14),
        DQ15            => dq(15),
        CLK             => mclk100_sig, 
        CKE             => SDRAM_CKE,
        A0              => daddr(0),
        A1              => daddr(1),
        A2              => daddr(2),
        A3              => daddr(3),
        A4              => daddr(4),
        A5              => daddr(5),
        A6              => daddr(6),
        A7              => daddr(7),
        A8              => daddr(8),
        A9              => daddr(9),
        A10             => daddr(10),
        A11             => daddr(11),
        WENeg           => wen,
        RASNeg          => rasn,
        CSNeg           => csn, 
        CASNeg          => casn
	);

   dq_out_dly <= SDRAM_DQO after 1 ns;
   daddr <= SDRAM_A after 1 ns;
   wen <= SDRAM_WE_N after 1 ns;
   csn <= SDRAM_CS_N after 1 ns;
   rasn <= SDRAM_RAS_N after 1 ns;
   casn <= SDRAM_CAS_N after 1 ns;

  dq <= dq_out_dly when SDRAM_DQENB_N = '0' else (others => 'Z');
  SDRAM_DQI <= dq after 2 ns;

env_1 : env port map (
     CLK => mclk_sig, 
     CLK100 => mclk100_sig, 
     RST_N => rst_n_sig,
          PCLK => pclk_sig,
    DCLK => DCLK,
    SCLK => SCLK,
          CamHsync => CamHsync_sig,
          CamVsync => CamVsync_sig,
     CamData => yin_sig );

MAIN_P: process

begin -- process MAIN_P

	wait;

end process ;  -- MAIN_P

end beh ;
