// Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

module DVI_CTRL(
  RST_N,
  DCLK,
  SCLK,
  disp_ena,
  disp_b,
  disp_g,
  disp_r,
  disp_hsync,
  disp_vsync,
  disp_control,
  data0p_h,
  data0p_l,
  data0n_h,
  data0n_l,
  data1p_h,
  data1p_l,
  data1n_h,
  data1n_l,
  data2p_h,
  data2p_l,
  data2n_h,
  data2n_l,
  clockp_h,
  clockp_l,
  clockn_h,
  clockn_l
);

input RST_N;
input DCLK;
input SCLK;
input disp_ena;
input[7:0] disp_b;
input[7:0] disp_g;
input[7:0] disp_r;
input disp_hsync;
input disp_vsync;
input[3:0] disp_control;
output data0p_h;
output data0p_l;
output data0n_h;
output data0n_l;
output data1p_h;
output data1p_l;
output data1n_h;
output data1n_l;
output data2p_h;
output data2p_l;
output data2n_h;
output data2n_l;
output clockp_h;
output clockp_l;
output clockn_h;
output clockn_l;

wire   RST_N;
wire   DCLK;
wire   SCLK;
wire   disp_ena;
wire  [7:0] disp_b;
wire  [7:0] disp_g;
wire  [7:0] disp_r;
wire   disp_hsync;
wire   disp_vsync;
wire  [3:0] disp_control;
wire   data0p_h;
wire   data0p_l;
wire   data0n_h;
wire   data0n_l;
wire   data1p_h;
wire   data1p_l;
wire   data1n_h;
wire   data1n_l;
wire   data2p_h;
wire   data2p_l;
wire   data2n_h;
wire   data2n_l;
wire   clockp_h;
wire   clockp_l;
wire   clockn_h;
wire   clockn_l;


wire [1:0] disp_vh;
wire [9:0] q_b;
wire [9:0] q_g;
wire [9:0] q_r;

tmds_encoder tmds_encoder_r(
  .clk(DCLK),
  //system clock
  .disp_ena(disp_ena),
  //display enable
  .control(disp_control[3:2] ),
  //C3, C2
  .d_in(disp_r),
  //8-bit data input
  .q_out(q_r));

tmds_encoder tmds_encoder_g(
  .clk(DCLK),
  //system clock
  .disp_ena(disp_ena),
  //display enable
  .control(disp_control[1:0] ),
  //C1, C0
  .d_in(disp_g),
  //8-bit data input
  .q_out(q_g));

tmds_encoder tmds_encoder_b(
  .clk(DCLK),
  //system clock
  .disp_ena(disp_ena),
  //display enable
  .control(disp_vh),
  //vsync, hsync
  .d_in(disp_b),
  //8-bit data input
  .q_out(q_b));

assign disp_vh = {disp_vsync,disp_hsync};
dvi_transmitter dvi_transmitter_1(
  .RST_N(RST_N),
  .DCLK(DCLK),
  .SCLK(SCLK),
  .data0_in(q_b),
  .data1_in(q_g),
  .data2_in(q_r),
  .data0p_h(data0p_h),
  .data0p_l(data0p_l),
  .data0n_h(data0n_h),
  .data0n_l(data0n_l),
  .data1p_h(data1p_h),
  .data1p_l(data1p_l),
  .data1n_h(data1n_h),
  .data1n_l(data1n_l),
  .data2p_h(data2p_h),
  .data2p_l(data2p_l),
  .data2n_h(data2n_h),
  .data2n_l(data2n_l),
  .clockp_h(clockp_h),
  .clockp_l(clockp_l),
  .clockn_h(clockn_h),
  .clockn_l(clockn_l));


endmodule
