// Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

module SDRAM_CTRL(
  CLK,
  CLK100M,
  RST_N,
  VgaHsync_EDGE,
  CamHsync_EDGE,
  CamVsync,
  CamVsync_EDGE,
  CamLineCount,
  CamPixCount4x,
  VgaLineCount,
  VgaPixCount,
  buf_RGB,
  latch0,
  latch1,
  latch2,
  latch3,
  vga_RGB_latch0,
  vga_RGB_latch1,
  vga_RGB_latch2,
  vga_RGB_latch3,
  winp0,
  winp1,
  winp2,
  winp3,
  SDRAM_DQENB_N,
  SDRAM_DQMH,
  SDRAM_DQML,
  SDRAM_CS_N,
  SDRAM_WE_N,
  SDRAM_RAS_N,
  SDRAM_CAS_N,
  SDRAM_CKE,
  SDRAM_DQI,
  SDRAM_DQO,
  SDRAM_A,
  SDRAM_BA
);

input CLK;
input CLK100M;
input RST_N;
input VgaHsync_EDGE;
input CamHsync_EDGE;
input CamVsync;
input CamVsync_EDGE;
input[8:0] CamLineCount;
input[15:0] CamPixCount4x;
input[8:0] VgaLineCount;
input[9:0] VgaPixCount;
input[15:0] buf_RGB;
output[15:0] latch0;
output[15:0] latch1;
output[15:0] latch2;
output[15:0] latch3;
input[15:0] vga_RGB_latch0;
input[15:0] vga_RGB_latch1;
input[15:0] vga_RGB_latch2;
input[15:0] vga_RGB_latch3;
output winp0;
output winp1;
output winp2;
output winp3;
output SDRAM_DQENB_N;
output SDRAM_DQMH;
output SDRAM_DQML;
output SDRAM_CS_N;
output SDRAM_WE_N;
output SDRAM_RAS_N;
output SDRAM_CAS_N;
output SDRAM_CKE;
input[15:0] SDRAM_DQI;
output[15:0] SDRAM_DQO;
output[11:0] SDRAM_A;
output[1:0] SDRAM_BA;

wire   CLK;
wire   CLK100M;
wire   RST_N;
wire   VgaHsync_EDGE;
wire   CamHsync_EDGE;
wire   CamVsync;
wire   CamVsync_EDGE;
wire  [8:0] CamLineCount;
wire  [15:0] CamPixCount4x;
wire  [8:0] VgaLineCount;
wire  [9:0] VgaPixCount;
wire  [15:0] buf_RGB;
wire  [15:0] latch0;
wire  [15:0] latch1;
wire  [15:0] latch2;
wire  [15:0] latch3;
wire  [15:0] vga_RGB_latch0;
wire  [15:0] vga_RGB_latch1;
wire  [15:0] vga_RGB_latch2;
wire  [15:0] vga_RGB_latch3;
reg    winp0;
reg    winp1;
reg    winp2;
reg    winp3;
reg    SDRAM_DQENB_N;
reg    SDRAM_DQMH;
reg    SDRAM_DQML;
reg    SDRAM_CS_N;
reg    SDRAM_WE_N;
reg    SDRAM_RAS_N;
reg    SDRAM_CAS_N;
reg    SDRAM_CKE;
wire  [15:0] SDRAM_DQI;
reg   [15:0] SDRAM_DQO;
reg   [11:0] SDRAM_A;
reg   [1:0] SDRAM_BA;


reg  [15:0] buf_RGB_dly1;
reg  [15:0] buf_RGB_dly2;
reg  [15:0] buf_RGB_dly3;
reg  [15:0] sdram_data0;
reg  [15:0] sdram_data1;
reg  [15:0] sdram_data2;
reg  [15:0] sdram_data3;
wire  buf_RGB_win;
reg  [4:0] MemState;
wire [4:0] Initial;
wire [4:0] MRS;
wire [4:0] BstWr;
wire [4:0] BstRd;
wire [4:0] PreChAll;
wire [4:0] doingPreChAll;
wire [4:0] doingMRS;
reg  [3:0] cmdcount;
reg  [4:0] count100m;
wire [15:0] sdram_data;
wire  csn;
wire  wrn;
wire  rasn;
wire  casn;
wire  dqm;
wire  dqenbn;
wire  cmdFound;
wire [1:0] baddr;
reg  [1:0] baddr_norm;
wire [11:0] coladdr;
wire [11:0] rowaddr;
reg  [15:0] pwrupcount;
wire  pwrupready;
wire  oddLine2x;
reg  [2:0] FrameCount;
wire [2:0] frame2write;
wire [2:0] frame2read;
wire [2:0] frame;
wire  SDRAM_DQENB_N_sig;
wire  SDRAM_DQMH_sig;
wire  SDRAM_DQML_sig;
wire  SDRAM_CS_N_sig;
wire  SDRAM_WE_N_sig;
wire  SDRAM_RAS_N_sig;
wire  SDRAM_CAS_N_sig;
wire  SDRAM_CKE_sig;
wire [15:0] SDRAM_DQO_sig;
wire [11:0] SDRAM_A_sig;
wire [1:0] SDRAM_BA_sig;
wire [8:0] line2write;
wire [8:0] line2read;
wire [8:0] linerw;
reg   rasn_norm;
reg   casn_norm;
reg  [11:0] rowaddr_norm;
reg  [11:0] coladdr_norm;
wire  dec14;
reg   dec15;
reg   dec16;
reg   dec17;
reg   dec18;
reg  [15:0] latchp0;
reg  [15:0] latchp1;
reg  [15:0] latchp2;
reg  [15:0] latchp3;
wire  rasn_norm_sig;
wire  casn_norm_sig;
wire [11:0] rowaddr_norm_sig;
wire [11:0] coladdr_norm_sig;
wire [1:0] baddr_norm_sig;
wire  count100m_clr;
reg   buf_RGB_win_dly1;
reg   buf_RGB_win_dly2;
wire  cmdready;

assign Initial = 5'b00000;
assign MRS = 5'b00010;
assign BstWr = 5'b00100;
assign BstRd = 5'b00101;
assign PreChAll = 5'b00001;
assign doingPreChAll = 5'b10001;
assign doingMRS = 5'b10010;

//-----------------------------------------------------------------
always@(posedge CLK or negedge RST_N)  begin // from CLK100M 2015/09/11
  if ((RST_N == 1'b0)  )  begin
    MemState <= Initial;
  end
  else  begin
    if ((MemState == Initial && pwrupready == 1'b1)  )  begin
      // go to precharge
      MemState <= PreChAll;
    end
    else if ((MemState == PreChAll)  )  begin
      MemState <= doingPreChAll;
    end
    else if ((MemState == doingPreChAll && cmdready == 1'b1)  )  begin
      // wait 300ns and go to MRS
      MemState <= MRS;
    end
    else if ((MemState == MRS)  )  begin
      MemState <= doingMRS;
    end
    else if ((MemState == doingMRS && cmdready == 1'b1)  )  begin
      // wait 300ns and go to burst write
      MemState <= BstWr;
    end
    else if ((MemState == BstWr && VgaHsync_EDGE == 1'b1 && oddLine2x == 1'b1)  )  begin
      // go to burst read
      MemState <= BstRd;
    end
    else if ((MemState == BstRd && VgaHsync_EDGE == 1'b1 && oddLine2x == 1'b0)  )  begin
      // go to burst write
      MemState <= BstWr;
    end
  end
end

//-----------------------------------------------------------------
assign cmdFound = (MemState == PreChAll || MemState == MRS)? 1'b1 : 1'b0;
//-----------------------------------------------------------------
always@(posedge CLK or negedge RST_N or posedge cmdFound)  begin
  if ((RST_N == 1'b0 || cmdFound == 1'b1)  )  begin
    cmdcount <= {4{1'b0}};
  end
  else  begin
    if ((cmdcount != 15)  )  begin
      cmdcount <= cmdcount + 1;
    end
  end
end

assign cmdready = (cmdcount == 15)? 1'b1 : 1'b0;
// 20ns * 15 = 300ns
//-----------------------------------------------------------------
assign buf_RGB_win = (CamPixCount4x[2:0]  == 3'b000)? 1'b1 : 1'b0;
//-----------------------------------------------------------------
always@(posedge CLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    buf_RGB_dly3 <= {16{1'b0}};
    buf_RGB_dly2 <= {16{1'b0}};
    buf_RGB_dly1 <= {16{1'b0}};
  end
  else  begin
    if ((CamPixCount4x[0]  == 1'b0)  )  begin
      buf_RGB_dly3 <= buf_RGB_dly2;
      buf_RGB_dly2 <= buf_RGB_dly1;
      buf_RGB_dly1 <= buf_RGB;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    sdram_data3 <= {16{1'b0}};
    sdram_data2 <= {16{1'b0}};
    sdram_data1 <= {16{1'b0}};
    sdram_data0 <= {16{1'b0}};
  end
  else  begin
    if ((buf_RGB_win == 1'b1)  )  begin
      sdram_data3 <= buf_RGB_dly3;
      sdram_data2 <= buf_RGB_dly2;
      sdram_data1 <= buf_RGB_dly1;
      sdram_data0 <= buf_RGB;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    buf_RGB_win_dly2 <= 1'b0;
    buf_RGB_win_dly1 <= 1'b0;
  end
  else  begin
    buf_RGB_win_dly2 <= buf_RGB_win_dly1;
    buf_RGB_win_dly1 <= buf_RGB_win;
  end
end

assign count100m_clr = (buf_RGB_win_dly1 == 1'b1 && buf_RGB_win_dly2 == 1'b0)? 1'b1 : 1'b0;
//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    count100m <= {5{1'b0}};
  end
  else  begin
    if ((count100m_clr == 1'b1)  )  begin
      count100m <= {5{1'b0}};
    end
    else  begin
      count100m <= count100m + 1;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    pwrupcount <= {16{1'b0}};
  end
  else  begin
    if ((pwrupcount != 10000)  )  begin
      // 20ns * 10000 = 200us
      pwrupcount <= pwrupcount + 1;
    end
  end
end

assign pwrupready = (pwrupcount == 10000)? 1'b1 : 1'b0;
//-----------------------------------------------------------------
assign oddLine2x = (CamPixCount4x < 1567)? 1'b0 : 1'b1;
//-----------------------------------------------------------------
always@(posedge CLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    FrameCount <= {3{1'b0}};
  end
  else  begin
    if ((CamVsync_EDGE == 1'b1)  )  begin

      FrameCount <= FrameCount + 1;
    end
  end
end

assign frame2write = FrameCount;
assign frame2read = (FrameCount == 0)? 3'b111 : (FrameCount == 1)? 3'b000 : (FrameCount == 2)? 3'b001 : (FrameCount == 3)? 3'b010 : (FrameCount == 4)? 3'b011 : (FrameCount == 5)? 3'b100 : (FrameCount == 6)? 3'b101 : 3'b110;
assign frame = (MemState == BstWr)? frame2write : frame2read;
//-----------------------------------------------------------------
assign line2write = CamLineCount[8:0] ;
//-----------------------------------------------------------------
assign line2read = VgaLineCount;
//-----------------------------------------------------------------
assign linerw = (MemState == BstWr)? line2write : line2read;
//-----------------------------------------------------------------
assign rasn_norm_sig = (dec16 == 1'b1)? 1'b0 : 1'b1;
assign casn_norm_sig = (count100m == 2)? 1'b0 : 1'b1;
assign rowaddr_norm_sig = {frame[0] ,linerw,VgaPixCount[9:8] };
assign coladdr_norm_sig = {4'b0100,VgaPixCount[7:2] ,2'b00};
assign baddr_norm_sig = frame[2:1] ;
//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    rasn_norm <= 1'b0;
    casn_norm <= 1'b0;
    rowaddr_norm <= {12{1'b0}};
    coladdr_norm <= {12{1'b0}};
    baddr_norm <= {2{1'b0}};
  end
  else  begin
    rasn_norm <= rasn_norm_sig;
    casn_norm <= casn_norm_sig;
    rowaddr_norm <= rowaddr_norm_sig;
    coladdr_norm <= coladdr_norm_sig;
    baddr_norm <= baddr_norm_sig;
  end
end

//-----------------------------------------------------------------
assign sdram_data = (count100m == 3)? sdram_data3 : (count100m == 4)? sdram_data2 : (count100m == 5)? sdram_data1 : (count100m == 6)? sdram_data0 : {16{1'b0}};
//-----------------------------------------------------------------
assign csn = 1'b0;
assign wrn = (MemState == BstWr && oddLine2x == 1'b0 && count100m == 3)? 1'b0 : 1'b1;

assign rasn = rasn_norm;

assign casn = casn_norm;

assign dqm = 1'b0;
assign baddr = baddr_norm;

assign dqenbn = (MemState == BstWr && oddLine2x == 1'b0 && count100m >= 3 && count100m <= 6)? 1'b0 : 1'b1;
assign rowaddr = rowaddr_norm;

assign coladdr = coladdr_norm;

//-----------------------------------------------------------------
assign SDRAM_CS_N_sig = (MemState == PreChAll || MemState == doingPreChAll)? 1'b0 : 
		(MemState == MRS)? 1'b0 : 
		(MemState == doingMRS)? 1'b0 : csn;

assign SDRAM_RAS_N_sig = (MemState == PreChAll || MemState == doingPreChAll)? 1'b0 : 
		(MemState == MRS)? 1'b0 : 
//		(MemState == doingMRS)? 1'b1 : rasn;
		(MemState == doingMRS)? 1'b0 : rasn; // same as MRS 2015/09/11

assign SDRAM_CAS_N_sig = (MemState == PreChAll)? 1'b1 : 
		(MemState == doingPreChAll)? 1'b0 : 
		(MemState == MRS)? 1'b0 : 
//		(MemState == doingMRS)? 1'b1 : casn;
		(MemState == doingMRS)? 1'b0 : casn; // same as MRS 2015/09/11

assign SDRAM_WE_N_sig = (MemState == PreChAll)? 1'b0 : 
		(MemState == doingPreChAll)? 1'b1 : 
		(MemState == MRS)? 1'b0 : 
//		(MemState == doingMRS)? 1'b1 : wrn;
		(MemState == doingMRS)? 1'b0 : wrn; // same as MRS 2015/09/11

assign SDRAM_A_sig = (MemState == PreChAll || MemState == doingPreChAll)? 12'b010000000000 : 
		(MemState == MRS)? 12'b000000100010 : 
//		(MemState == doingMRS)? 12'b000000000000 : 
		(MemState == doingMRS)? 12'b000000100010 : // same as MRS 2015/09/11
		(count100m == 1 || count100m == 9)? rowaddr : 
		(count100m == 3 || count100m == 11)? coladdr : {12{1'b0}};

assign SDRAM_BA_sig = (MemState == MRS)? 2'b00 : baddr;

assign SDRAM_CKE_sig = 1'b1;
assign SDRAM_DQENB_N_sig = dqenbn;
assign SDRAM_DQO_sig = sdram_data;
assign SDRAM_DQMH_sig = 1'b0;
assign SDRAM_DQML_sig = 1'b0;
//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    SDRAM_BA <= {2{1'b0}};
    SDRAM_A <= {12{1'b0}};
    SDRAM_DQO <= {16{1'b0}};
    SDRAM_DQMH <= 1'b0;
    SDRAM_DQML <= 1'b0;
    SDRAM_CS_N <= 1'b0;
    SDRAM_WE_N <= 1'b0;
    SDRAM_RAS_N <= 1'b0;
    SDRAM_CAS_N <= 1'b0;
    SDRAM_CKE <= 1'b0;
    SDRAM_DQENB_N <= 1'b0;
  end
  else  begin
    SDRAM_BA <= SDRAM_BA_sig;
    SDRAM_A <= SDRAM_A_sig;
    SDRAM_DQO <= SDRAM_DQO_sig;
    SDRAM_DQMH <= SDRAM_DQMH_sig;
    SDRAM_DQML <= SDRAM_DQML_sig;
    SDRAM_CS_N <= SDRAM_CS_N_sig;
    SDRAM_WE_N <= SDRAM_WE_N_sig;
    SDRAM_RAS_N <= SDRAM_RAS_N_sig;
    SDRAM_CAS_N <= SDRAM_CAS_N_sig;
    SDRAM_CKE <= SDRAM_CKE_sig;
    SDRAM_DQENB_N <= SDRAM_DQENB_N_sig;
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    latchp0 <= {16{1'b0}};
  end
  else  begin
    if ((count100m == 6)  )  begin
      latchp0 <= SDRAM_DQI;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    latchp1 <= {16{1'b0}};
  end
  else  begin
    if ((count100m == 7)  )  begin
      latchp1 <= SDRAM_DQI;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    latchp2 <= {16{1'b0}};
  end
  else  begin
    if ((count100m == 8)  )  begin
      latchp2 <= SDRAM_DQI;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    latchp3 <= {16{1'b0}};
  end
  else  begin
    if ((count100m == 9)  )  begin
      latchp3 <= SDRAM_DQI;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    winp0 <= 1'b0;
  end
  else  begin
    if (((count100m == 6 || count100m == 7)   && oddLine2x == 1'b1)  )  begin
      winp0 <= 1'b1;
    end
    else  begin
      winp0 <= 1'b0;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    winp1 <= 1'b0;
  end
  else  begin
    if (((count100m == 8 || count100m == 9)   && oddLine2x == 1'b1)  )  begin
      winp1 <= 1'b1;
    end
    else  begin
      winp1 <= 1'b0;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    winp2 <= 1'b0;
  end
  else  begin
    if (((count100m == 10 || count100m == 11)   && oddLine2x == 1'b1)  )  begin
      winp2 <= 1'b1;
    end
    else  begin
      winp2 <= 1'b0;
    end
  end
end

//-----------------------------------------------------------------
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    winp3 <= 1'b0;
  end
  else  begin
    if (((count100m == 12 || count100m == 13)   && oddLine2x == 1'b1)  )  begin
      winp3 <= 1'b1;
    end
    else  begin
      winp3 <= 1'b0;
    end
  end
end

//-----------------------------------------------------------------
assign dec14 = (count100m == 14)? 1'b1 : 1'b0;
always@(posedge CLK100M or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    dec18 <= 1'b0;
    dec17 <= 1'b0;
    dec16 <= 1'b0;
    dec15 <= 1'b0;
  end
  else  begin
    dec18 <= dec17;
    dec17 <= dec16;
    dec16 <= dec15;
    dec15 <= dec14;
  end
end

//-----------------------------------------------------------------
assign latch0 = latchp0;
assign latch1 = latchp1;
assign latch2 = latchp2;
assign latch3 = latchp3;

endmodule
