// Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

module dvi_transmitter(
  RST_N,
  DCLK,
  SCLK,
  data0_in,
  data1_in,
  data2_in,
  data0p_h,
  data0p_l,
  data0n_h,
  data0n_l,
  data1p_h,
  data1p_l,
  data1n_h,
  data1n_l,
  data2p_h,
  data2p_l,
  data2n_h,
  data2n_l,
  clockp_h,
  clockp_l,
  clockn_h,
  clockn_l
);

input RST_N;
input DCLK;
input SCLK;
input[9:0] data0_in;
input[9:0] data1_in;
input[9:0] data2_in;
output data0p_h;
output data0p_l;
output data0n_h;
output data0n_l;
output data1p_h;
output data1p_l;
output data1n_h;
output data1n_l;
output data2p_h;
output data2p_l;
output data2n_h;
output data2n_l;
output clockp_h;
output clockp_l;
output clockn_h;
output clockn_l;

wire   RST_N;
wire   DCLK;
wire   SCLK;
wire  [9:0] data0_in;
wire  [9:0] data1_in;
wire  [9:0] data2_in;
reg    data0p_h;
reg    data0p_l;
reg    data0n_h;
reg    data0n_l;
reg    data1p_h;
reg    data1p_l;
reg    data1n_h;
reg    data1n_l;
reg    data2p_h;
reg    data2p_l;
reg    data2n_h;
reg    data2n_l;
reg    clockp_h;
reg    clockp_l;
reg    clockn_h;
reg    clockn_l;


reg  [9:0] data0_latch;
reg  [9:0] data1_latch;
reg  [9:0] data2_latch;
reg  [9:0] data0_ser;
reg  [9:0] data1_ser;
reg  [9:0] data2_ser;
reg  [9:0] clock_ser;
reg  [4:0] start_sig;

//------------------------------------------------------------ 
always@(posedge DCLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    data0_latch <= {10{1'b0}};
    data1_latch <= {10{1'b0}};
    data2_latch <= {10{1'b0}};
  end
  else  begin
    data0_latch <= data0_in;
    data1_latch <= data1_in;
    data2_latch <= data2_in;
  end
end

//------------------------------------------------------------ 
always@(posedge SCLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    start_sig <= 5'b00001;
  end
  else  begin
    start_sig <= {start_sig[0] ,start_sig[4:1] };
  end
end

//------------------------------------------------------------ 
always@(posedge SCLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    data0_ser <= {10{1'b0}};
    data1_ser <= {10{1'b0}};
    data2_ser <= {10{1'b0}};
    clock_ser <= {10{1'b0}};
  end
  else  begin
    if ((start_sig[0]  == 1'b1)  )  begin
      data0_ser <= data0_latch;
      data1_ser <= data1_latch;
      data2_ser <= data2_latch;
      clock_ser <= 10'b0000011111;
    end
    else  begin
      data0_ser <= {2'b00,data0_ser[9:2] };
      data1_ser <= {2'b00,data1_ser[9:2] };
      data2_ser <= {2'b00,data2_ser[9:2] };
      clock_ser <= {2'b00,clock_ser[9:2] };
    end
  end
end

//------------------------------------------------------------ 
always@(posedge SCLK or negedge RST_N)  begin
  if ((RST_N == 1'b0)  )  begin
    data0p_h <= 1'b0;
    data0p_l <= 1'b0;
    data0n_h <= 1'b0;
    data0n_l <= 1'b0;
    data1p_h <= 1'b0;
    data1p_l <= 1'b0;
    data1n_h <= 1'b0;
    data1n_l <= 1'b0;
    data2p_h <= 1'b0;
    data2p_l <= 1'b0;
    data2n_h <= 1'b0;
    data2n_l <= 1'b0;
    clockp_h <= 1'b0;
    clockp_l <= 1'b0;
    clockn_h <= 1'b0;
    clockn_l <= 1'b0;
  end
  else  begin
    data0p_h <= data0_ser[0] ;
    data0p_l <= data0_ser[1] ;
    data0n_h <=  ~data0_ser[0] ;
    data0n_l <=  ~data0_ser[1] ;
    data1p_h <= data1_ser[0] ;
    data1p_l <= data1_ser[1] ;
    data1n_h <=  ~data1_ser[0] ;
    data1n_l <=  ~data1_ser[1] ;
    data2p_h <= data2_ser[0] ;
    data2p_l <= data2_ser[1] ;
    data2n_h <=  ~data2_ser[0] ;
    data2n_l <=  ~data2_ser[1] ;
    clockp_h <= clock_ser[0] ;
    clockp_l <= clock_ser[1] ;
    clockn_h <=  ~clock_ser[0] ;
    clockn_l <=  ~clock_ser[1] ;
  end
end


endmodule
