`timescale 1ns/1ps

module tb_VideoProc;
	wire CLK, CLK100M, RST_N, DCLK, SCLK;
	wire [7:0] CamData;

wire SDRAM_CLK;
wire SDRAM_DQENB_N;
wire SDRAM_DQMH;
wire SDRAM_DQML;
wire SDRAM_CS_N;
wire SDRAM_WE_N;
wire SDRAM_RAS_N;
wire SDRAM_CAS_N;
wire SDRAM_CKE;
wire [15:0] SDRAM_DQI;
wire [15:0] SDRAM_DQO;
wire [11:0] SDRAM_A;
wire [1:0] SDRAM_BA;

wire [15:0] dq ;
wire wen ;
wire rasn ;
wire csn ;
wire casn ;
wire dqenbn;
wire [1:0] baddr ;
wire [11:0] addr ;
wire [15:0] dq_out_dly ;

	VideoProcCore VideoProcCore_inst(
		.CLK(CLK),
		.CLK100M(CLK100M),
		.RST_N(RST_N),
	.XCLK(),
	.CamHsync(CamHsync),
	.CamVsync(CamVsync),
	.PCLK(PCLK),
  .CamData(CamData),
  .SDRAM_DQENB_N(SDRAM_DQENB_N),
  .SDRAM_DQMH(SDRAM_DQMH),
  .SDRAM_DQML(SDRAM_DQML),
  .SDRAM_CS_N(SDRAM_CS_N),
  .SDRAM_WE_N(SDRAM_WE_N),
  .SDRAM_RAS_N(SDRAM_RAS_N),
  .SDRAM_CAS_N(SDRAM_CAS_N),
  .SDRAM_CKE(SDRAM_CKE),
  .SDRAM_DQI(SDRAM_DQI),
  .SDRAM_DQO(SDRAM_DQO),
  .SDRAM_A(SDRAM_A),
  .SDRAM_BA(SDRAM_BA),
  .SDRAM_CLK(SDRAM_CLK),
		.DCLK(DCLK),
		.SCLK(SCLK),

		.data0p_h(),
		.data0p_l(),
		.data0n_h(),
		.data0n_l(),

		.data1p_h(),
		.data1p_l(),
		.data1n_h(),
		.data1n_l(),

		.data2p_h(),
		.data2p_l(),
		.data2n_h(),
		.data2n_l(),

		.clockp_h(),
		.clockp_l(),
		.clockn_h(),
		.clockn_l()	); 

	env env_inst(
		.CLK(CLK),
		.CLK100M(CLK100M),
		.RST_N(RST_N),
	.PCLK(PCLK),
	.DCLK(DCLK),
	.SCLK(SCLK),
	.CamHsync(CamHsync),
	.CamVsync(CamVsync),
	.CamData(CamData)
	);

mt48lc4m16 mt48lc4m16_inst(
     .A11      (addr[11]      ),
     .A10      (addr[10]      ),
     .A9       (addr[9]       ),
     .A8       (addr[8]       ),
     .A7       (addr[7]       ),
     .A6       (addr[6]       ),
     .A5       (addr[5]       ),
     .A4       (addr[4]       ),
     .A3       (addr[3]       ),
     .A2       (addr[2]       ),
     .A1       (addr[1]       ),
     .A0       (addr[0]       ),

     .DQ15      (dq[15]      ),
     .DQ14      (dq[14]      ),
     .DQ13      (dq[13]      ),
     .DQ12      (dq[12]      ),
     .DQ11      (dq[11]      ),
     .DQ10      (dq[10]      ),
     .DQ9       (dq[9]       ),
     .DQ8       (dq[8]       ),
     .DQ7       (dq[7]       ),
     .DQ6       (dq[6]       ),
     .DQ5       (dq[5]       ),
     .DQ4       (dq[4]       ),
     .DQ3       (dq[3]       ),
     .DQ2       (dq[2]       ),
     .DQ1       (dq[1]       ),
     .DQ0       (dq[0]       ),

     .BA0  (baddr[0]  ),
     .BA1  (baddr[1]  ),
     .DQMH (SDRAM_DQMH ),
     .DQML (SDRAM_DQML ),
     .CLK  (CLK100M  ),
     .CKE  (SDRAM_CKE  ),
     .WENeg (wen ) ,
     .RASNeg (rasn ),
     .CSNeg  (csn  ),
     .CASNeg (casn )
     );

   assign #1 dq_out_dly = SDRAM_DQO;
   assign #1 addr = SDRAM_A;
   assign #1 baddr = SDRAM_BA;
   assign #1 wen = SDRAM_WE_N;
   assign #1 csn = SDRAM_CS_N;
   assign #1 rasn = SDRAM_RAS_N;
   assign #1 casn = SDRAM_CAS_N;
   assign #1 dqenbn = SDRAM_DQENB_N;

  assign dq = dqenbn == 0 ? dq_out_dly : 16'bz;
  assign #2 SDRAM_DQI = dqenbn == 1 ? dq : 16'bz;

endmodule
