// Designed by Toshio Iwata at DIGITALFILTER.COM 2015/09/22

module tmds_encoder(
  clk,
  disp_ena,
  control,
  d_in,
  q_out
);

input clk;
//system clock
input disp_ena;
//display enable
input[1:0] control;
//C1, C0
input[7:0] d_in;
//8-bit data input
output[9:0] q_out;

wire   clk;
wire   disp_ena;
wire  [1:0] control;
wire  [7:0] d_in;
reg   [9:0] q_out;

//10-bit encoder output

reg  [8:0] q_m;
//internal data
wire [7:0] one_count_d_in;
// RANGE 0 TO 8;         --number of ones in d_in
wire [7:0] one_count_q_m;
// INTEGER RANGE 0 TO 8;         --number of ones in q_m
wire signed [7:0] diff_q_m;
// INTEGER RANGE -8 TO 8;        --number of ones minus the number of zeros in q_m
reg signed [7:0] disparity;
// INTEGER RANGE -16 TO 15;      --disparity
wire  sum_a0;
wire  sum_b0;
wire  sum_c0;
wire  sum_d0;
wire  sum_e0;
wire  sum_f0;
wire  sum_g0;
wire  cout_a0;
wire  cout_b0;
wire  cout_c0;
wire  cout_d0;
wire  cout_e0;
wire  cout_f0;
wire  cout_g0;
wire  sum_a1;
wire  sum_b1;
wire  sum_c1;
wire  sum_d1;
wire  sum_e1;
wire  sum_f1;
wire  sum_g1;
wire  cout_a1;
wire  cout_b1;
wire  cout_c1;
wire  cout_d1;
wire  cout_e1;
wire  cout_f1;
wire  cout_g1;
wire  dummy_low;

assign dummy_low = 1'b0;

//count the ones in d_in
fulladd4dvi COMP_A0(
  (d_in[0] ),
  (d_in[1] ),
  (d_in[2] ),
  (sum_a0),
  (cout_a0));

fulladd4dvi COMP_B0(
  (d_in[3] ),
  (d_in[4] ),
  (d_in[5] ),
  (sum_b0),
  (cout_b0));

fulladd4dvi COMP_C0(
  (d_in[6] ),
  (d_in[7] ),
  (dummy_low),
  (sum_c0),
  (cout_c0));

fulladd4dvi COMP_D0(
  (sum_a0),
  (sum_b0),
  (sum_c0),
  (sum_d0),
  (cout_d0));

fulladd4dvi COMP_E0(
  (cout_a0),
  (cout_b0),
  (cout_c0),
  (sum_e0),
  (cout_e0));

fulladd4dvi COMP_F0(
  (cout_d0),
  (sum_e0),
  (dummy_low),
  (sum_f0),
  (cout_f0));

fulladd4dvi COMP_G0(
  (cout_f0),
  (cout_e0),
  (dummy_low),
  (sum_g0),
  (cout_g0));

assign one_count_d_in[0]  = sum_d0;
assign one_count_d_in[1]  = sum_f0;
assign one_count_d_in[2]  = sum_g0;
assign one_count_d_in[3]  = cout_g0;
assign one_count_d_in[4]  = 1'b0;
assign one_count_d_in[5]  = 1'b0;
assign one_count_d_in[6]  = 1'b0;
assign one_count_d_in[7]  = 1'b0;

//process interval data to minimize transitions
always@(d_in or q_m or one_count_d_in)  begin
  if ((one_count_d_in > 4 || (one_count_d_in == 4 && d_in[0]  == 1'b0)  )  )  begin
    q_m[0]  <= d_in[0] ;
    q_m[1] <= !(q_m[0] ^ d_in[1]);
    q_m[2] <= !(q_m[1] ^ d_in[2]);
    q_m[3] <= !(q_m[2] ^ d_in[3]);
    q_m[4] <= !(q_m[3] ^ d_in[4]);
    q_m[5] <= !(q_m[4] ^ d_in[5]);
    q_m[6] <= !(q_m[5] ^ d_in[6]);
    q_m[7] <= !(q_m[6] ^ d_in[7]);
    q_m[8]  <= 1'b0;
  end
  else  begin
    q_m[0]  <= d_in[0] ;
    q_m[1] <= q_m[0] ^ d_in[1];
    q_m[2] <= q_m[1] ^ d_in[2];
    q_m[3] <= q_m[2] ^ d_in[3];
    q_m[4] <= q_m[3] ^ d_in[4];
    q_m[5] <= q_m[4] ^ d_in[5];
    q_m[6] <= q_m[5] ^ d_in[6];
    q_m[7] <= q_m[6] ^ d_in[7];
    q_m[8]  <= 1'b1;
  end
end

//count the ones in q_m
fulladd4dvi COMP_A1(
  q_m[0] ,
  q_m[1] ,
  q_m[2] ,
  sum_a1,
  cout_a1);

fulladd4dvi COMP_B1(
  q_m[3] ,
  q_m[4] ,
  q_m[5] ,
  sum_b1,
  cout_b1);

fulladd4dvi COMP_C1(
  q_m[6] ,
  q_m[7] ,
  dummy_low,
  sum_c1,
  cout_c1);

fulladd4dvi COMP_D1(
  sum_a1,
  sum_b1,
  sum_c1,
  sum_d1,
  cout_d1);

fulladd4dvi COMP_E1(
  cout_a1,
  cout_b1,
  cout_c1,
  sum_e1,
  cout_e1);

fulladd4dvi COMP_F1(
  cout_d1,
  sum_e1,
  dummy_low,
  sum_f1,
  cout_f1);

fulladd4dvi COMP_G1(
  cout_f1,
  cout_e1,
  dummy_low,
  sum_g1,
  cout_g1);

assign one_count_q_m[0]  = sum_d1;
assign one_count_q_m[1]  = sum_f1;
assign one_count_q_m[2]  = sum_g1;
assign one_count_q_m[3]  = cout_g1;
assign one_count_q_m[4]  = 1'b0;
assign one_count_q_m[5]  = 1'b0;
assign one_count_q_m[6]  = 1'b0;
assign one_count_q_m[7]  = 1'b0;
assign diff_q_m = one_count_q_m + one_count_q_m - 8;

//determine output and new disparity
always@(posedge clk)  begin
  if ((disp_ena == 1'b1)  )  begin
    if ((disparity == 0 || one_count_q_m == 4)  )  begin
      if ((q_m[8]  == 1'b0)  )  begin
        q_out <= { ~q_m[8] ,q_m[8] , ~q_m[7:0] };
        disparity <= disparity - diff_q_m;
      end
      else  begin
        q_out <= { ~q_m[8] ,q_m[8:0] };
        disparity <= disparity + diff_q_m;
      end
    end
    else  begin
      if (((disparity > 0 && one_count_q_m > 4)   || (disparity < 0 && one_count_q_m < 4)  )  )  begin
        q_out <= {1'b1,q_m[8] , ~q_m[7:0] };
        if ((q_m[8]  == 1'b0)  )  begin
          disparity <= disparity - diff_q_m;
        end
        else  begin
          disparity <= disparity - diff_q_m + 2;
        end
      end
      else  begin
        q_out <= {1'b0,q_m[8:0] };
        if ((q_m[8]  == 1'b0)  )  begin
          disparity <= disparity + diff_q_m - 2;
        end
        else  begin
          disparity <= disparity + diff_q_m;
        end
      end
    end
  end
  else  begin
    case (control)
    2'b00:  begin
      q_out <= 10'b1101010100;
    end
    2'b01:  begin
      q_out <= 10'b0010101011;
    end
    2'b10:  begin
      q_out <= 10'b0101010100;
    end
    2'b11:  begin
      q_out <= 10'b1010101011;
    end
    default:  begin
    end
    endcase
    disparity <= {8{1'b0}};
  end
end


endmodule
